@@ -67,6 +67,7 @@ enum fio_fallocate_mode {
 	FIO_FALLOCATE_POSIX	= 2,
 	FIO_FALLOCATE_KEEP_SIZE	= 3,
 	FIO_FALLOCATE_NATIVE	= 4,
+	FIO_FALLOCATE_TRUNCATE	= 5,
 };
 
 /*
@@ -89,7 +90,6 @@ struct fio_file {
 	 */
 	unsigned int major, minor;
 	int fileno;
-	unsigned long long bs;
 	char *file_name;
 
 	/*
@@ -104,6 +104,9 @@ struct fio_file {
 	 * Zoned block device information. See also zonemode=zbd.
 	 */
 	struct zoned_block_device_info *zbd_info;
+	/* zonemode=zbd working area */
+	uint32_t min_zone;	/* inclusive */
+	uint32_t max_zone;	/* exclusive */
 
 	/*
 	 * Track last end and last start of IO for a given data direction
